from urllib.parse import quote

import requests
import json

from roulette_russe.Scraper import Scraper


REFERER_URL = "https://www.instagram.com/"
BASE_URL = "https://www.instagram.com/"
LOGIN_URL = BASE_URL + "accounts/login/ajax/"
USER_AGENT_STORIES = "Instagram 123.0.0.21.114 (iPhone; CPU iPhone OS 11_4 like Mac OS X; en_US; en-US; scale=2.00; 750x1334) AppleWebKit/605.1.15"
USER_AGENT = "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.87 Safari/537.36"


class InstagramScraper(Scraper):
    def __init__(self, input_config, output_datasets, username, password, rr=None):
        super(InstagramScraper, self).__init__(input_config, output_datasets, rr)
        self.session = requests.Session()
        self.session.cookies.set('ig_pr', '1')
        self.session.headers.update({"Referer": REFERER_URL, "user-agent": USER_AGENT_STORIES})
        req = self.session.get(BASE_URL)
        self.session.headers.update({'X-CSRFToken': req.cookies['csrftoken']})
        login_data = {'username': username, 'password': password}
        login = self.session.post(LOGIN_URL, data=login_data, allow_redirects=True)
        self.session.headers.update({'X-CSRFToken': login.cookies['csrftoken']})
        self.cookies = login.cookies
        login_text = json.loads(login.text)
        if login_text.get('authenticated') and login.status_code == 200:
            self.session.headers.update({'user-agent': USER_AGENT})
            print('OK')

    def query_main_infos(self, cookies, username):
        req = self.session.get("https://www.instagram.com/" + username, cookies=cookies).text
        return {
            "description": req.split('user":{"biography":"')[1].split('","')[0],
            "number_following": req.split('"edge_follow":{"count":')[1].split('},"')[0],
            "number_followers": req.split('"edge_followed_by":{"count":')[1].split('},"')[0],
            "full_name": req.split('"full_name":"')[2].split('","')[0],
            "username": req.split('"username":"')[2].split('","')[0],
            "twitter_internal_id": req.split('"id":"')[2].split('","')[0],
            "is_private_profile": req.split('"is_private":')[2].split(',"')[0],
        }

    def _query_followings(self, cookies, id, end_cursor=''):
        QUERY_FOLLOWINGS = BASE_URL + 'graphql/query/?query_hash=c56ee0ae1f89cdbd1c89e2bc6b8f3d18&variables={0}'
        QUERY_FOLLOWINGS_VARS = '{{"id":"{0}","first":50,"after":"{1}"}}'
        params = QUERY_FOLLOWINGS_VARS.format(id, end_cursor)
        req = self.session.get(QUERY_FOLLOWINGS.format(params), cookies=cookies).text
        if req is not None:
            payload = json.loads(req)['data']['user']['edge_follow']
            if payload:
                end_cursor = payload['page_info']['end_cursor']
                followings = []
                for node in payload['edges']:
                    followings.append(node['node']['username'])
                return followings, end_cursor
        return None, None

    def _query_followers(self,cookies, id, end_cursor=''):
        req = self.session.get(
            'https://www.instagram.com/graphql/query/?query_hash=c76146de99bb02f6415203be841dd25a&variables={"id":"' + id + '","include_reel":true,"fetch_mutual":true,"first":24,"after":"' + end_cursor + '"}',
            cookies=cookies).text
        if req is not None:
            payload = json.loads(req)['data']['user']['edge_followed_by']
            if payload:
                end_cursor = payload['page_info']['end_cursor']
                followers = []
                for node in payload['edges']:
                    followers.append(node['node']['username'])
                return followers, end_cursor
        return None, None

    def _query_commenters_of_post(self, cookies, post_short_code, end_cursor=''):
        QUERY_COMMENTERS = BASE_URL + 'graphql/query/?query_hash=33ba35852cb50da46f5b5e889df7d159&variables={0}'
        QUERY_COMMENTERS_VARS = '{{"shortcode":"{0}","include_reel":true,"first":12,"after":"{1}"}}'
        params = QUERY_COMMENTERS_VARS.format(post_short_code, end_cursor)
        req = self.session.get(QUERY_COMMENTERS.format(params), cookies=cookies).text
        if req is not None:
            payload = json.loads(req)['data']['shortcode_media']['edge_media_to_comment']
            if payload:
                end_cursor = payload['page_info']['end_cursor']
                commenters = []
                for node in payload['edges']:
                    commenters.append(node['node']['owner']['username'])
                return commenters, end_cursor
        return None, None

    def _query_likers_of_post(self, cookies, post_short_code, end_cursor=''):
        QUERY_LIKERS = BASE_URL + 'graphql/query/?query_hash=d5d763b1e2acf209d62d22d184488e57&variables={0}'
        QUERY_LIKERS_VARS = '{{"shortcode":"{0}","include_reel":true,"first":12,"after":"{1}"}}'
        params = QUERY_LIKERS_VARS.format(post_short_code, end_cursor)
        req = self.session.get(QUERY_LIKERS.format(params), cookies=cookies).text
        if req is not None:
            payload = json.loads(req)['data']['shortcode_media']['edge_liked_by']
            if payload:
                end_cursor = payload['page_info']['end_cursor']
                likers = []
                for node in payload['edges']:
                    likers.append(node['node']['username'])
                return likers, end_cursor
        return None, None

    def _query_posts(self, cookies, id, end_cursor=''):
        req = self.session.get(
            'https://www.instagram.com/graphql/query/?query_hash=bfa387b2992c3a52dcbe447467b4b771&variables={"id":"' + id + '","first":24,"after":"' + end_cursor + '"}',
            cookies=cookies).text

        if req is not None:
            payload = json.loads(req)['data']['user']['edge_owner_to_timeline_media']
            if payload:
                end_cursor = payload['page_info']['end_cursor']
                posts = []
                for node in payload['edges']:
                    posts.append({
                        "is_video": node['node']['is_video'],
                        "short_code": node['node']['shortcode'],
                        "internal_id": node['node']['id'],
                        "link": "https://www.instagram.com/p/" + node['node']['shortcode'],
                        "comments_number": node['node']['edge_media_to_comment']['count'],
                        "likes_number": node['node']['edge_media_preview_like']['count'],
                        "timestamp": node['node']['taken_at_timestamp']
                    })
                return posts, end_cursor
            return None, None

    def query_posts(self, cookies, id):
        posts, end_cursor = self._query_followers(cookies, id)

        if posts:
            while True:
                for post in posts:
                    yield post
                if end_cursor:
                    posts, end_cursor = self._query_posts(cookies, id, end_cursor)
                else:
                    return

    def query_posts_likers(self, cookies, post_short_code):
        likers, end_cursor = self._query_likers_of_post(cookies, post_short_code)

        if likers:
            while True:
                for liker in likers:
                    yield liker
                if end_cursor:
                    likers, end_cursor = self._query_likers_of_post(cookies, post_short_code, end_cursor)
                else:
                    return

    def query_posts_commenters(self, cookies, post_short_code):
        commenters, end_cursor = self._query_likers_of_post(cookies, post_short_code)

        if commenters:
            while True:
                for commenter in commenters:
                    yield commenter
                if end_cursor:
                    commenters, end_cursor = self._query_commenters_of_post(cookies, post_short_code, end_cursor)
                else:
                    return

    def query_followers(self, cookies, id):
        followers, end_cursor = self._query_followers(cookies, id)

        if followers:
            while True:
                for follower in followers:
                    yield follower
                if end_cursor:
                    followers, end_cursor = self._query_followers(cookies, id, end_cursor)
                else:
                    return

    def query_followings(self, cookies, id):
        followings, end_cursor = self._query_followings(cookies, id)

        if followings:
            while True:
                for following in followings:
                    yield following
                if end_cursor:
                    followings, end_cursor = self._query_followings(cookies, id, end_cursor)
                else:
                    return

    def scrap(self):
        tmp_list_profiles=[]
        tmp_list_posts=[]
        tmp_list_followers = []
        tmp_list_following = []
        tmp_list_commenters = []
        tmp_list_likers = []
        for account in self.input_config['accounts']:
            profile_data = self.query_main_infos(cookies=self.cookies, username=account)
            tmp_list_profiles.append(profile_data)
        self.output_datasets['profiles'] = self.output_datasets['profiles'].append(tmp_list_profiles, ignore_index=True)
        if 'posts' in self.output_datasets:
            for account in tmp_list_profiles:
                print(account['twitter_internal_id'])
                posts_data = self.query_posts(self.cookies, account['twitter_internal_id'])
                print(list(posts_data))
                tmp_list_posts.append(list(posts_data))
            self.output_datasets['posts'] = self.output_datasets['posts'].append(tmp_list_posts, ignore_index=True)
        if 'followers' in tmp_list_profiles:
            for account in self.output_datasets['profiles']:
                followers_data = self.query_followers(self.cookies, account['twitter_internal_id'])
                tmp_list_followers.append(followers_data)
            self.output_datasets['followers'] = self.output_datasets['followers'].append(tmp_list_followers, ignore_index=True)
        if 'following' in tmp_list_profiles:
            for account in self.output_datasets['profiles']:
                following_data = self.query_followings(self.cookies, account['twitter_internal_id'])
                tmp_list_following.append(following_data)
            self.output_datasets['following'] = self.output_datasets['following'].append(tmp_list_following, ignore_index=True)
        if 'commenters' in self.output_datasets:
            for post in tmp_list_posts:
                commenters_data = self.query_posts_commenters(self.cookies, post['short_code'])
                tmp_list_commenters.append(commenters_data)
            self.output_datasets['commenters'] = self.output_datasets['commenters'].append(tmp_list_commenters, ignore_index=True)
        if 'likers' in self.output_datasets:
            for post in tmp_list_posts:
                likers_data = self.query_posts_likers(self.cookies, post['short_code'])
                tmp_list_likers.append(likers_data)
            self.output_datasets['likers'] = self.output_datasets['likers'].append(tmp_list_likers, ignore_index=True)


    def quality_check(self):
        pass



import requests
from bs4 import BeautifulSoup
from roulette_russe.Scraper import Scraper


class Choosemycompany(Scraper):
    def __init__(self, input_config, output_datasets, rr=None):
        super(Choosemycompany, self).__init__(input_config, output_datasets, rr)

    def get_pride(self, soup):
        return soup.findAll("a", {"id": "progress-fierte"})[0].findAll(
            "div", {"class": "progress-bar"})[0].attrs["aria-valuenow"]

    def get_company_logo_link(self, soup):
        if len(soup.findAll("a", {"class": "company-logo"})[0].findAll("img")) == 0:
            return ''
        return soup.findAll("a", {"class": "company-logo"})[0].findAll("img")[0].attrs['src']

    def get_professional_development(self, soup):
        return soup.findAll("a", {"id": "progress-developpement-professionnel"})[0].findAll("div", {
            "class": "progress-bar"})[0].attrs["aria-valuenow"]

    def get_motivation_and_management(self, soup):
        return soup.findAll("a", {"id": "progress-motivation-management"})[0].findAll("div", {"class": "progress-bar"})[
            0].attrs["aria-valuenow"]

    def get_fun_and_pleasure(self, soup):
        return soup.findAll("a", {"id": "progress-motivation-management"})[0].findAll("div", {"class": "progress-bar"})[
            0].attrs["aria-valuenow"]

    def get_employees_score(self, soup):
        return soup.findAll("a", {"class": "companypage-menu-section notation-block employees selected"})[0].findAll(
            "span", {"class": "notation rating"})[0].findAll("span")[0].text.replace('/ 5', '')

    def get_employees_raitings_count(self, soup):
        return \
            soup.findAll("div", {"class": "notation-legend text-right"})[0].text.replace(' ', '').replace('\n',
                                                                                                          '').split(
                'avis')[0].replace('\xa0', '0')

    def get_trainees_score(self, soup):
        return soup.findAll("a", {"class": "companypage-menu-section notation-block trainees"})[0].findAll("span", {
            "class": "notation rating"})[0].text.replace('/ 5', '')

    def get_trainees_raitings_count(self, soup):
        return \
            soup.findAll("div", {"class": "notation-legend text-right"})[1].text.replace(' ', '').replace('\n',
                                                                                                          '').split(
                'avis')[0].replace('\xa0', '0')

    def get_candidates_score(self, soup):
        return soup.findAll("a", {"class": "companypage-menu-section notation-block candidates"})[0].findAll("span", {
            "class": "notation rating"})[0].text.replace('/ 5', '')

    def get_candidates_raitings_count(self, soup):
        return \
            soup.findAll("div", {"class": "notation-legend text-right"})[2].text.replace(' ', '').replace('\n',
                                                                                                          '').split(
                'avis')[0].replace('\xa0', '0')

    def get_clients_score(self, soup):
        return soup.findAll("a", {"class": "companypage-menu-section notation-block clients"})[0].findAll("span", {
            "class": "notation rating"})[0].text.replace('/ 5', '')

    def get_clients_raitings_count(self, soup):
        return \
            soup.findAll("div", {"class": "notation-legend text-right"})[3].text.replace(' ', '').replace('\n',
                                                                                                          '').split(
                'avis')[0].replace('\xa0', '0')

    def get_simulating_env_rating(self, soup):
        return soup.findAll("a", {"id": "progress-environnement-de-travail"})[0].findAll(
            "div", {"class": "progress-bar"})[0].attrs["aria-valuenow"]

    def get_recognition_and_rewards_raiting(self, soup):
        return soup.findAll("a", {"id": "progress-salaire-reconnaissance"})[0].findAll(
            "div", {"class": "progress-bar"})[0].attrs["aria-valuenow"]

    def get_description(self, soup):
        return soup.findAll('div', {'class': 'activities-content'})[0].text

    def scrape_one_company(self, link):
        r = requests.get(link, proxies=self.get_proxies()).text
        soup = BeautifulSoup(r, 'html.parser')
        r = requests.get(link.replace('/salaries', '/infos'), proxies=self.get_proxies()).text
        soup_infos = BeautifulSoup(r, 'html.parser')
        return {
            'company_name': soup.findAll("h1")[0].text,
            'description': self.get_description(soup_infos),
            'company_logo': self.get_company_logo_link(soup),
            'global_score': soup.findAll("span", {"class": "average"})[0].text,
            'ratings_count': soup.findAll("span", {"class": "count"})[0].text,
            'employees_score': self.get_employees_score(soup),
            'employees_raitings_count': self.get_employees_raitings_count(soup),
            'trainees_score': self.get_trainees_score(soup),
            'trainees_raitings_count': self.get_trainees_raitings_count(soup),
            'candidates_score': self.get_candidates_score(soup),
            'candidates_raitings_count': self.get_candidates_raitings_count(soup),
            'clients_score': self.get_clients_score(soup),
            'clients_raitings_count': self.get_clients_raitings_count(soup),
            'stimulating_environment': self.get_simulating_env_rating(soup),
            'recognition_and_rewards': self.get_recognition_and_rewards_raiting(soup),
            'pride': self.get_pride(soup),
            'professional_development': self.get_professional_development(soup),
            'motivation_and_management': self.get_motivation_and_management(soup),
            'fun_and_pleasure': self.get_fun_and_pleasure(soup),
        }

    def scrap(self):
        tmp_list = []
        for link in self.input_config['links']:
            tmp_list.append(self.scrape_one_company(link))
        self.output_datasets['profiles'] = self.output_datasets['profiles'].append(tmp_list, ignore_index=True)
        print(self.output_datasets['profiles'].head())

    def quality_check(self):
        pass



import dns.resolver
import random
import boto3
import json


class RouletteRusse(object):
    """
    A class used to represent a Roulette Russe

    ...
    Attributes
    ----------
    proxy_domain : str
        the domain name that binds the proxy IP addresses in A rows
    proxy_username : str, optional
        the ursername of the proxy
    proxy_password : str, optional
        the password of the proxy. If not provided then the proxy is not protected by a username/password mechanism
    proxy_port : int, optional
        the proxy port. If not provided then the default value 3128 will be used
    """
    def __init__(self, aws_access_key_id: str, aws_secret_access_key: str):
        session = boto3.Session(
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key
        )
        s3 = session.client('s3')
        s3.download_file('hubfinanceconfiguration', 'RouletteRusseConfig.json', 'RouletteRusseConfig.json')
        with open('RouletteRusseConfig.json') as config_file:
            rr_config_file = json.load(config_file)
        self.ip_list = dns.resolver.resolve(rr_config_file['proxy_domain_name'], 'A')
        self.proxy_username = rr_config_file['username']
        self.proxy_password = rr_config_file['password']
        self.proxy_port = rr_config_file['proxy_port']
        self.config = rr_config_file

    def getRandomIPAddress(self) -> str:
        return self.ip_list[random.randint(0, len(self.ip_list)-1)]

    def getProxyCredentials(self):
        return {
            'username': self.proxy_username,
            'password': self.proxy_password
        }

    def getProxyConnectionString(self) -> str:
        if self.proxy_username is None or self.proxy_password is None:
            return 'http://' + str(self.getRandomIPAddress()) + ':' + str(self.proxy_port)
        return 'http://' + self.proxy_username + ':' + self.proxy_password + '@' + str(
            self.getRandomIPAddress()) + ':' + str(self.proxy_port)

    def getRandomMobileUserAgent(self):
        mobile_ua_list = self.config["useragents"]["mobile"]
        return mobile_ua_list[random.randint(0, len(mobile_ua_list)-1)]

    def getRandomLaptopUserAgent(self):
        laptop_ua_list = self.config["useragents"]["laptop"]
        return laptop_ua_list[random.randint(0, len(laptop_ua_list)-1)]



from urllib.parse import quote

import requests

from roulette_russe.Scraper import Scraper


class TwitterScraper(Scraper):
    guest_token = ''
    guest_token_number_of_uses = 180

    def __init__(self, input_config, output_datasets, rr=None):
        super(TwitterScraper, self).__init__(input_config, output_datasets, rr)

    def get_guest_token(self):
        ua = self.get_laptop_user_agent()
        print("UA used to renew guest token for twitter: ", ua)
        self.guest_token_number_of_uses += 1
        if self.guest_token_number_of_uses > 30:
            r = requests.get('https://twitter.com/CNBC', proxies=self.get_proxies(), headers={
                'User-Agent': ua
            })
            self.guest_token_number_of_uses = 1
            self.guest_token = r.text.split('decodeURIComponent("gt=')[1].split(';')[0]
        return self.guest_token

    def get_num_id_from_username(self, username):
        print(username)
        r = requests.get('https://api.twitter.com/1.1/search/typeahead.json?q=' + username, proxies=self.get_proxies(),
                         headers={
                             'User-Agent': self.get_laptop_user_agent(),
                             'x-guest-token': self.get_guest_token(),
                             'authorization': 'Bearer AAAAAAAAAAAAAAAAAAAAANRILgAAAAAAnNwIzUejRCOuH5E6I8xnZz4puTs%3D1Zv7ttfk8LF81IUq16cHjhLTvJu4FA33AGWWjCpTnA'
                         })
        return str(r.json()['users'][0]['id'])

    def get_profile_data(self, twitter_numerical_id):
        while 1:
            try:
                r = requests.get('https://api.twitter.com/2/timeline/profile/' + twitter_numerical_id + '.json',
                                 headers={
                                     'User-Agent': self.get_mobile_user_agent(),
                                     'x-guest-token': self.get_guest_token(),
                                     'authorization': 'Bearer AAAAAAAAAAAAAAAAAAAAANRILgAAAAAAnNwIzUejRCOuH5E6I8xnZz4puTs%3D1Zv7ttfk8LF81IUq16cHjhLTvJu4FA33AGWWjCpTnA'
                                 })
                break
            except IndexError as e:
                print('TWITTER TRIED TO BLOCK ME')
                continue
        profile_data = r.json()['globalObjects']['users'][twitter_numerical_id]
        return {
            'twitter_internal_id': profile_data['id_str'],
            'username': profile_data['screen_name'],
            'company_tagline': profile_data['description'],
            'logo_link': profile_data['profile_image_url_https'],
            'followers_number': profile_data['followers_count'],
            'following_number': profile_data['friends_count'],
            'date_joined': profile_data['created_at'],
            'number_of_tweets': profile_data['statuses_count']
        }

    def get_tweets_data(self, twitter_numerical_id, account_name):
        tweets_list = []
        cursor_query = ''
        while 1:
            try:
                r = requests.get(
                    quote(
                        'https://api.twitter.com/2/timeline/profile/' + twitter_numerical_id + '.json?include_profile_interstitial_type=1&include_blocking=1&include_blocked_by=1&include_followed_by=1&include_want_retweets=1&include_mute_edge=1&include_can_dm=1&include_can_media_tag=1&skip_status=1&cards_platform=Web-12&include_cards=1&include_ext_alt_text=true&include_quote_count=true&include_reply_count=1&tweet_mode=extended&include_entities=true&include_user_entities=true&include_ext_media_color=true&include_ext_media_availability=true&send_error_codes=true&simple_quoted_tweet=true&include_tweet_replies=false&userId=' + twitter_numerical_id + '&count=20' + cursor_query,
                        safe='/:?=&'), proxies=self.get_proxies(),
                    headers={
                        'User-Agent': self.get_mobile_user_agent(),
                        'x-guest-token': self.get_guest_token(),
                        'authorization': 'Bearer AAAAAAAAAAAAAAAAAAAAANRILgAAAAAAnNwIzUejRCOuH5E6I8xnZz4puTs%3D1Zv7ttfk8LF81IUq16cHjhLTvJu4FA33AGWWjCpTnA'
                    })
            except IndexError as e:
                print('TWITTER TRIED TO BLOCK ME')
                continue
            tweets = r.json()['globalObjects']['tweets']
            if tweets == {}:
                break
            entries_length = len(r.json()['timeline']['instructions'][0]['addEntries']['entries']) - 1
            cursor_query = '&cursor=' + \
                           r.json()['timeline']['instructions'][0]['addEntries']['entries'][entries_length]['content'][
                               'operation']['cursor']['value']
            for k in tweets:
                tweets_list.append({
                    'twitter_internal_id': twitter_numerical_id,
                    'tweet_date': tweets[k]['created_at'],
                    'tweet_url': 'https://twitter.com/' + account_name + '/status/' + k,
                    'comments_number': tweets[k]['reply_count'],
                    'retweets_number': tweets[k]['retweet_count'],
                    'likes_number': tweets[k]['favorite_count']
                })
            if len(tweets_list) > 1000:
                return tweets_list
        return tweets_list

    def scrap(self):
        tmp_list = []
        if 'profiles' in self.output_datasets:
            for account in self.input_config['accounts']:
                profile_data = self.get_profile_data(self.get_num_id_from_username(account))
                tmp_list.append(profile_data)
            self.output_datasets['profiles'] = self.output_datasets['profiles'].append(tmp_list, ignore_index=True)
            print(self.output_datasets['profiles'].head())

        if 'tweets' in self.output_datasets:
            for account in self.input_config['accounts']:
                self.output_datasets['tweets'] = self.output_datasets['tweets'].append(
                    self.get_tweets_data(self.get_num_id_from_username(account), account),
                    ignore_index=True)

    def quality_check(self):
        pass



import requests
from bs4 import BeautifulSoup
from roulette_russe.Scraper import Scraper


class GlassdoorScraper(Scraper):
    def __init__(self, input_config, output_datasets, rr=None):
        super(GlassdoorScraper, self).__init__(input_config, output_datasets, rr)

    def get_company_logo(self, soup):
        return soup.findAll("span",{"class": "sqLogo"})[0].findAll("img")[0].attrs['src']

    def get_company_name(self, soup):
        return soup.findAll("span",{"id": "DivisionsDropdownComponent"})[0].text

    def get_company_description(self, soup):
        return soup.findAll("div", {"class": "empDescription"})[0].text

    def get_number_of_reviews(self, soup):
        return soup.findAll("a", {"class":"reviews"})[0].findAll("span", {"class": "num"})[0].text

    def get_number_of_jobs(self, soup):
        return soup.findAll("a", {"class":"jobs"})[0].findAll("span", {"class": "num"})[0].text

    def get_number_of_salaries(self, soup):
        return soup.findAll("a", {"class":"salaries"})[0].findAll("span", {"class": "num"})[0].text

    def get_number_of_interviews(self, soup):
        return soup.findAll("a", {"class":"interviews"})[0].findAll("span", {"class": "num"})[0].text

    def get_number_of_benefits(self, soup):
        return soup.findAll("a", {"class":"benefits"})[0].findAll("span", {"class": "num"})[0].text

    def get_number_of_photos(self, soup):
        return soup.findAll("a", {"class":"photos"})[0].findAll("span", {"class": "num"})[0].text

    def get_overall_rating(self, text):
        return text.split('overallRating":')[1].split(',')[0]

    def get_ceoRating(self, text):
        return text.split('ceoRating":')[1].split(',')[0]

    def get_ceoRating_count(self, text):
        return text.split('ceoRatingsCount":')[1].split(',')[0]

    def get_career_opportunities_rating(self, text):
        return text.split('careerOpportunitiesRating":')[1].split(',')[0]

    def get_work_life_balance(self, text):
        return text.split('workLifeBalanceRating":')[1].split(',')[0]

    def get_business_outlook_rating(self, text):
        return text.split('businessOutlookRating":')[1].split(',')[0]

    def get_senior_management_rating(self, text):
        return text.split('seniorManagementRating":')[1].split(',')[0]

    def get_compensation_and_benefits_rating(self, text):
        return text.split('compensationAndBenefitsRating":')[1].split(',')[0]

    def get_culture_and_values_rating(self, text):
        return text.split('cultureAndValuesRating":')[1].split(',')[0]

    def scrape_profile(self, link):
        r = requests.get(link, headers={
            'User-Agent': self.get_laptop_user_agent()
        }, proxies=self.get_proxies()).text

        soup = BeautifulSoup(r, 'html.parser')
        reviews_link = "https://www.glassdoor.fr" + soup.findAll("a", {"class": "reviews"})[0].attrs['href']
        r_reviews = requests.get(reviews_link, headers={
            'User-Agent': self.get_laptop_user_agent()
        }, proxies=self.get_proxies()).text
        return {
            "company_name": self.get_company_name(soup),
            "company_logo": self.get_company_logo(soup),
            "number_of_reviews": self.get_number_of_reviews(soup),
            "number_of_jobs": self.get_number_of_jobs(soup),
            "number_of_salaries": self.get_number_of_salaries(soup),
            "number_of_interviews": self.get_number_of_interviews(soup),
            "number_of_benefits": self.get_number_of_benefits(soup),
            "number_of_photos": self.get_number_of_photos(soup),
            "overall_rating": self.get_overall_rating(r_reviews),
            "ceo_rating": self.get_ceoRating(r_reviews),
            "ceo_rating_count": self.get_ceoRating_count(r_reviews),
            "career_opportunities_rating": self.get_career_opportunities_rating(r_reviews),
            "work_life_balance_rating": self.get_work_life_balance(r_reviews),
            "business_outlook_rating": self.get_business_outlook_rating(r_reviews),
            "senior_management_rating": self.get_senior_management_rating(r_reviews),
            "compensation_and_benefits_rating": self.get_compensation_and_benefits_rating(r_reviews),
            "culture_and_values_rating": self.get_culture_and_values_rating(r_reviews)
        }

    def scrap(self):
        tmp_list = []
        if 'profiles' in self.output_datasets:
            for link in self.input_config['links']:
                profile_data = self.scrape_profile(link)
                tmp_list.append(profile_data)
            self.output_datasets['profiles'] = self.output_datasets['profiles'].append(tmp_list, ignore_index=True)

    def quality_check(self):
        pass


import pandas as pd
import boto3
from botocore.exceptions import ClientError
from abc import abstractmethod, ABC
import logging
import os


class Scraper(ABC):
    def __init__(self, input_config: dict, output_datasets: list, roulette_russe=None):
        self.output_datasets = dict()
        for i in range(len(output_datasets)):
            self.output_datasets[output_datasets[i]] = pd.DataFrame()
        self.input_config = input_config
        self.roulette_russe = roulette_russe

    def get_mobile_user_agent(self):
        if self.roulette_russe is None:
            return 'Mozilla/5.0 (iPhone; CPU iPhone OS 13_2_3 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/13.0.3 Mobile/15E148 Safari/604.1'
        return self.roulette_russe.getRandomMobileUserAgent()

    def get_laptop_user_agent(self):
        if self.roulette_russe is None:
            return 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.61 Safari/537.36'
        return self.roulette_russe.getRandomLaptopUserAgent()

    def get_proxies(self):
        if self.roulette_russe is None:
            return {}
        else:
            return {
                "http": self.roulette_russe.getProxyConnectionString(),
                "https": self.roulette_russe.getProxyConnectionString(),
            }

    @abstractmethod
    def scrap(self):
        pass

    def to_csv(self, path, dataset_index):
        self.output_datasets[dataset_index].to_csv(path, index=False)

    def to_parquet(self, path, dataset_index):
        self.output_datasets[dataset_index].to_parquet(path, index=False)

    @abstractmethod
    def quality_check(self):
        pass

    def to_s3(self, aws_access_key_id: str, aws_secret_access_key: str, bucket_name: str, file_name: str, dataset_index: int):
        local_file_name = file_name.split('/')[len(file_name.split('/'))-1]
        session = boto3.Session(
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key
        )
        s3 = session.client('s3')
        self.to_parquet(local_file_name, dataset_index)
        try:
            s3.upload_file(local_file_name, bucket_name, file_name)
            os.remove(local_file_name)
        except ClientError as e:
            logging.error(e)
            os.remove(local_file_name)
            return False
        return True


import requests

from roulette_russe.Scraper import Scraper


class YoutubeScraper(Scraper):
    def __init__(self, input_config, output_datasets, rr=None):
        super(YoutubeScraper, self).__init__(input_config, output_datasets, rr)

    def extract_channel_title(self, payload):
        return payload['metadata']['channelMetadataRenderer']['title']

    def extract_channel_description(self, payload):
        return payload['metadata']['channelMetadataRenderer']['description']

    def extract_channel_keywords(self, payload):
        return payload['metadata']['channelMetadataRenderer']['keywords'].split(' ')

    def extract_channel_logo(self, payload):
        return payload['metadata']['channelMetadataRenderer']['avatar']['thumbnails'][0]['url']

    def extract_number_of_subscribers(self, payload):
        text_n_subscribers = payload['header']['c4TabbedHeaderRenderer']['subscriberCountText']['simpleText'].split(
            '\xa0')
        if len(text_n_subscribers) == 3:
            return text_n_subscribers[0:2]
        else:
            return text_n_subscribers[0]

    def extract_number_of_views(self, payload):
        result = 0
        for tab in payload['contents']['twoColumnBrowseResultsRenderer']['tabs']:
            if (tab['tabRenderer']['title'] == "À propos") or (tab['tabRenderer']['title'] == "About"):
                result = int(tab['tabRenderer']['content']['sectionListRenderer']['contents'][0]['itemSectionRenderer'][
                                 'contents'][0]['channelAboutFullMetadataRenderer']['viewCountText'][
                                 'simpleText'].replace('\xa0vues', '').replace('\u202f', '').replace('\xa0views',
                                                                                                     '').replace(',',
                                                                                                                 '').replace(
                    ' views', ''))
                break
        return result

    def scrape_channel_profile(self, channel_id):
        r = requests.get('https://youtube.com/c/' + channel_id + "/about?pbj=1", headers={
            "User-Agent": self.get_laptop_user_agent(),
            "X-YouTube-Client-Name": "1",
            "X-YouTube-Client-Version": "2.20200806.01.01"
        }, proxies=self.get_proxies()).json()
        payload = r[1]['response']
        return {
            "channel_title": self.extract_channel_title(payload),
            "channel_description": self.extract_channel_description(payload),
            "channel_keywords": self.extract_channel_keywords(payload),
            "channel_logo": self.extract_channel_logo(payload),
            "number_of_subscribers": self.extract_number_of_subscribers(payload),
            "number_of_views": self.extract_number_of_views(payload),
        }

    def scrap(self):
        tmp_list = []
        if 'profiles' in self.output_datasets:
            for account in self.input_config['accounts']:
                profile_data = self.scrape_channel_profile(account)
                tmp_list.append(profile_data)
            self.output_datasets['profiles'] = self.output_datasets['profiles'].append(tmp_list, ignore_index=True)

    def quality_check(self):
        pass



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()
setuptools.setup(
    name="roulette_russe",
    version="0.0.50",
    author="Mohamed Amjad LASRI",
    author_email="amjadepot@gmail.com",
    description="Library to write your scrapers hassle free.",
    url="https://github.com/Homestr/roulette_russe",
    keywords=['SCRAPING', 'ROULETTE_RUSSE', 'CRAWLING'],
    packages=setuptools.find_packages(),
    install_requires=[
        'vaex',
        'dnspython',
        'boto3',
        'beautifulsoup4'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)